/**
 * Title:        Can Monitor Client Connector <p>
 * Description:  <p>
 * Copyright:    Copyright (c) Jaap van Bergeijk<p>
 * Company:      Wageningen University<p>
 * @author Jaap van Bergeijk
 * @version 1.0
 */
package com.wur.can;

import java.io.*;
import java.net.*;
import javax.swing.*;

public class CanMonitorClientConnector extends Thread
{
  static final long KEEP_ALIVE_MESSAGE_INTERVAL = 5000;   // milliseconds
  private boolean bStopThread = true;

  private String serverName;
  private CanMonitorClientStatusConnection canStatusConnection = null;
  private CanMonitorClientFramesConnection canFramesConnection = null;

  private JTextArea textLogWindow = null;

  public CanMonitorClientConnector()
  {
//    serverName = new String("10.31.7.100");
    serverName = new String("192.168.0.42");
    canStatusConnection = new CanMonitorClientStatusConnection(this);
    canFramesConnection = new CanMonitorClientFramesConnection(this);
  }

  public void setServerName(String serverName)
  {
    this.serverName = serverName;
  }

  public String getServerName()
  {
    return this.serverName;
  }

  public void run()
  {
    bStopThread = false;
    while (!bStopThread)
    {
      boolean bInterrupted = false;
      try
      {
        sleep(KEEP_ALIVE_MESSAGE_INTERVAL);
      }
      catch (java.lang.InterruptedException ie)
      {
        bInterrupted = true;
      }

      if (!bInterrupted)
      {
        // send status message
        canStatusConnection.sendKeepAliveMessage();
      }
    }

    if (canStatusConnection.isConnected()) canStatusConnection.stopThread();
    if (canFramesConnection.isConnected()) canFramesConnection.stopThread();
  }

  public void stopThread()
  {
    bStopThread = true;
  }

  public void startStatusConnection()
  {
    canStatusConnection.start();
  }

  public void startFramesConnection()
  {
    canFramesConnection.start();
  }

  public void openFramesConnection()
  {
    canStatusConnection.openFramesConnection();
  }

  public void closeFramesConnection()
  {
    canStatusConnection.closeFramesConnection();
  }

  public void newCanFrameReceived(CanFrame canFrame)
  {
//    System.out.println("new frame");
    if (textLogWindow != null)
    {
      String str = canFrame.ToString();
      textLogWindow.append(str);
      textLogWindow.append("\n");
    }
  }

  public void addTextLogWindow(JTextArea t)
  {
    textLogWindow = t;
  }

  public void sendTest ()
  {
    canFramesConnection.sendTest();
  }
}