/**
 * Title:        Can Monitor Graphical Client<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Jaap van Bergeijk<p>
 * Company:      Wageningen University<p>
 * @author Jaap van Bergeijk
 * @version 1.0
 */
package com.wur.can;

import java.io.*;
import java.net.*;

public class CanMonitorClientFramesConnection extends Thread
{
  static final int FRAMES_PORT = 8;
  private boolean bStopThread = true;

  private CanMonitorClientConnector connector = null;

  private ServerSocket serverSocket = null;
  private Socket framesSocket = null;
  private DataInputStream dIn = null;
  private DataOutputStream dOut = null;
  private boolean bConnected = false;

  public CanMonitorClientFramesConnection(CanMonitorClientConnector c)
  {
    this.connector = c;
  }

  public boolean isConnected()
  {
    return bConnected;
  }

  public void run()
  {
    try
    {
      // start server socket for exchange of CanFrames
      serverSocket = new ServerSocket(FRAMES_PORT);

//----------
      // block and wait for new connection
      framesSocket = serverSocket.accept();

      try
      {
        dIn = new DataInputStream(framesSocket.getInputStream());
        dOut = new DataOutputStream(framesSocket.getOutputStream());

        int itemNumber = 0;
        boolean bExtended;
        int id;
        int length, bytesToRead;
        byte[] bytes = new byte[16];
        bExtended = true;
        id = 0;
        length = 0;
        bytesToRead = 0;

        bConnected = true;
        bStopThread = false;

        System.out.println("Frames socket opened");

        while (!bStopThread)
        {
          switch (itemNumber)
          {
            case 0:
              bExtended = dIn.readBoolean();
              itemNumber++;
              break;
            case 1:
              id = dIn.readInt();
              itemNumber++;
              break;
            case 2:
              length = dIn.readInt();
              bytesToRead = length;
              itemNumber++;
              break;
            case 3:
              if (bytesToRead > 0)
              {
                bytes[length-bytesToRead] = dIn.readByte();
                bytesToRead--;
              }
              else
              {
                // at this point a new CanFrame was received
                CanFrame cf = new CanFrame(bExtended,id,length,bytes);
                connector.newCanFrameReceived(cf);
                itemNumber = 0;
              }
              break;
          }
        }

        System.out.println("Frames socket opened end");
      }
      catch (IOException ioe)
      {
        System.out.println(ioe);
        ioe.printStackTrace();
      }
      finally
      {
        try
        {
          bConnected = false;
          dOut.close();
          dIn.close();
          framesSocket.close();
          System.out.println("Frames socket closed");
        }
        catch (IOException e)
        {
          System.out.println(e);
          e.printStackTrace();
        }
      }
    }
    catch(Exception e){}
    finally
    {
      try
      {
        serverSocket.close();
      }
      catch (IOException i)
      {
        System.out.println(i);
      }
    }
  }

  public void stopThread()
  {
    System.out.println("Can Frames stop");
    bStopThread = true;
  }

  public void sendTest ()
  {
    boolean bExtended = true;
    int id = 0x18EAFFFE;
    int length = 3;
    byte[] bytes = new byte[3];
    bytes[0] = 0x00;
    bytes[1] = (byte)0xEE;
    bytes[2] = 0x00;

    try
    {
      dOut.writeBoolean(bExtended);
      dOut.writeInt(id);
      dOut.writeInt(length);
      dOut.write(bytes,0,3);
    }
    catch (java.io.IOException e)
    {
      System.out.println(e);
    }
  }
}