/**
 * Title:        Can Monitor Graphical Client<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Jaap van Bergeijk<p>
 * Company:      Wageningen University<p>
 * @author Jaap van Bergeijk
 * @version 1.0
 */
package com.wur.can;

import java.io.*;
import java.net.*;

public class CanMonitorClientStatusConnection extends Thread
{
  static final int STATUS_PORT = 7;
  private boolean bStopThread = true;

  private CanMonitorClientConnector connector = null;
  private Socket statusSocket = null;
  private InputStream statusIn = null;
  private OutputStream statusOut = null;
  private boolean bConnected = false;

  public CanMonitorClientStatusConnection(CanMonitorClientConnector c)
  {
    this.connector = c;
  }

  public boolean isConnected()
  {
    return bConnected;
  }

  public void run()
  {
    try
    {
      statusSocket = new Socket(InetAddress.getByName(connector.getServerName()), STATUS_PORT);
      statusIn = statusSocket.getInputStream();
      statusOut = statusSocket.getOutputStream();

      System.out.println("Status socket opened");

      byte b;
      bConnected = true;
      bStopThread = false;
      while (!bStopThread)
      {
        // blocked read
        b = (byte) statusIn.read();
        System.out.write(b);
      }
    }
    catch (IOException ioe)
    {
      System.out.println(ioe);
      ioe.printStackTrace();
    }
    finally
    {
      try
      {
        bConnected = false;
        statusIn.close();
        statusOut.close();
        statusSocket.close();
        System.out.println("Status socket closed");
      }
      catch (IOException e)
      {
        System.out.println(e);
        e.printStackTrace();
      }
    }
  }

  public void stopThread()
  {
    System.out.println("Can Status stop");
    try
    {
      statusOut.write('d');
    }
    catch (IOException e)
    {
      System.out.println(e);
      e.printStackTrace();
    }
    bStopThread = true;
  }

  public void openFramesConnection()
  {
    if (bConnected)
    {
      try
      {
        statusOut.write('o');
      }
      catch (IOException e)
      {
        System.out.println(e);
        e.printStackTrace();
      }
    }
  }

  public void closeFramesConnection()
  {
    if (bConnected)
    {
      try
      {
        statusOut.write('c');
      }
      catch (IOException e)
      {
        System.out.println(e);
        e.printStackTrace();
      }
    }
  }

  public void sendKeepAliveMessage()
  {
    if (bConnected)
    {
      try
      {
        statusOut.write('a');
      }
      catch (IOException e)
      {
        System.out.println(e);
        e.printStackTrace();
      }
    }
  }
}